/******************************************************************************
 * Nano MasterMind Vers.3 
 * per Arduino Nano 
 * MMXXIII by maodnt
 * from Italy
 * 
 * Display LCD1602 I2C    >    Arduino Nano
   -------------------        -------------
           GND                     GND
           VCC                     5V
           SDA                     A4
           SCL                     A5
 *  
 * Tastierino a Membrana 4X4 >  Arduino Nano
    -------------------        -------------
          1                        D2
          2                        D3
          3                        D4
          4                        D5
          5                        D6
          6                        D7
          7                        D8
          8                        D9
 * 
 * Codici casuali da 3 a 6 cifre (numeri possibili nel codice 0123456789)
 * TASTI:
 * Incremento complessità codice con tasto A (da 3 a 6)
 * Conferma tentativo codice con tasto B
 * Correzione inserimento (backspace) con tasto C
 * Abbandona gioco e rivelazione codice con tasto D
 * Tasto * visualizza i Record per 6 secondi
 * Tast # non utilizzato
 * FEEDBACK CODICE (00) : prima cifra > Numeri corretti nel posto corretto
 *                        seconda cifra > Numeri corretti nella posiz. sbagliata
 ********************************************************************************                        
 */
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <Keypad.h>
#include <EEPROM.h>

bool flashScoresScreen = false;

// Dichiarazioni delle variabili di punteggio
int Score3 = 50;
int Score4 = 50;
int Score5 = 50;
int Score6 = 50;

// Indirizzi di memoria nella EEPROM
int addressScore3 = 0;
int addressScore4 = sizeof(int);  // Offset per separare i valori nella EEPROM
int addressScore5 = 2 * sizeof(int);
int addressScore6 = 3 * sizeof(int);

const byte rows = 4; // righe
const byte columns = 4; // colonne

char Keys[rows][columns] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};

byte rowPins[rows] = {2, 3, 4, 5}; // pin delle righe della tastiera
byte colPins[columns] = {6, 7, 8, 9}; // pin delle colonne della tastiera

LiquidCrystal_I2C lcd(0x27, 16, 2);
Keypad keypad = Keypad(makeKeymap(Keys), rowPins, colPins, rows, columns);

// Icone personalizzate
byte Sad[8] = { 
  0b00000,
  0b01010,
  0b00000,
  0b00000,
  0b01110,
  0b10001,
  0b00000,
  0b00000
};

byte Check[8] = { 
  0b00000,
  0b00001,
  0b00010,
  0b00100,
  0b10100,
  0b01000,
  0b00000,
  0b00000
};

byte Null[8] = { 
  0b00000,
  0b10001,
  0b01010,
  0b00100,
  0b01010,
  0b10001,
  0b00000,
  0b00000
};

byte Smile[8] = { 
  0b00000,
  0b01010,
  0b00000,
  0b00000,
  0b10001,
  0b01110,
  0b00000,
  0b00000
};

char code[6];  // Codice segreto di lunghezza variabile
int maxCodeLength = 6;  // Lunghezza massima del codice segreto
int currentCodeLength = 3;  // Lunghezza iniziale del codice segreto

char guess[6];  // Tentativo corrente
int currentDigit = 0;  // Posizione corrente durante l'inserimento del tentativo
bool inputMode = false;  // Stato dell'input

void generateSecretCode() {
  // Genera un numero casuale all'inizio del programma ma non lo utilizza come codice segreto
  // Risolve un problema del codice casuale all'avvio di Arduino che altrimenti sarebbe sempre identico
  randomSeed(analogRead(0));
  random(10);

  for (int i = 0; i < currentCodeLength; i++) {
    code[i] = random(10) + '0';  // Genera una cifra casuale
  }
  code[currentCodeLength] = '\0';  // Aggiungi il terminatore di stringa

  Serial.print("Codice segreto: ");
  Serial.println(code);  // Stampa il codice segreto su Serial Monitor (DEBUG)
}

void increaseCodeLength() {
  currentCodeLength++;

  // Se raggiunge 7 cifre, torna a 3 cifre
  if (currentCodeLength > 6) {
    currentCodeLength = 3;
  }

  generateSecretCode();
  displayCode();
}

void displayCode() {
  lcd.clear();
  lcd.setCursor(0, 0);  // Posizione (0, 0)
  lcd.print("FEEDBACK: ");
  lcd.setCursor(11, 0);  // Posizione (11, 0)
  lcd.write(byte(1));
  lcd.setCursor(12, 0);  // Posizione (12, 0)
  lcd.write((byte)2);
  lcd.setCursor(0, 1);  // Posizione (0, 1)
  lcd.print("Guess: ");
  for (int i = 0; i < currentCodeLength; i++) {
    lcd.print("-");
  }
}

void displayIndication(int correctPlace, int wrongPlace) {
  lcd.setCursor(14, 0);  // Posizione (12, 0)
  
  if (correctPlace == 0) {
    lcd.print("0");
  } else {
    lcd.print(correctPlace);
  }
  lcd.setCursor(15, 0);
  lcd.print(wrongPlace);

  if (correctPlace == 0 && wrongPlace == 0) {
    lcd.print(" ");
  }
}

void displayGuess() {
  lcd.setCursor(0, 1);
  lcd.print("Guess: ");
  for (int i = 0; i < currentDigit; i++) {
    lcd.print(guess[i]);
  }
  for (int i = currentDigit; i < currentCodeLength; i++) {
    lcd.print("-");
  }
}

void startNewGame() {
  generateSecretCode();
  displayCode();
}

void setup() {
  lcd.begin();
  lcd.backlight();
  Serial.begin(9600);
  lcd.createChar(0, Sad);
  lcd.createChar(1, Check);
  lcd.createChar(2, Null);
  lcd.createChar(3, Smile); 

  // Carica i valori della EEPROM nelle variabili di punteggio solo se i dati sono validi
  if (EEPROM.read(addressScore3) != 255) {
    EEPROM.get(addressScore3, Score3);
  }
  if (EEPROM.read(addressScore4) != 255) {
    EEPROM.get(addressScore4, Score4);
  }
  if (EEPROM.read(addressScore5) != 255) {
    EEPROM.get(addressScore5, Score5);
  }
  if (EEPROM.read(addressScore6) != 255) {
    EEPROM.get(addressScore6, Score6);
  }

  // Welcome
  lcd.clear();
  lcd.setCursor(0, 0);  // Posizione (0, 0)
  lcd.print("nano Master Mind");
  lcd.setCursor(0, 1);  // Posizione (0, 1)
  lcd.print("v3********maodnt"); 
  
  bool StartGame = false;
  while (!StartGame) {
    // Verifica se un tasto stato premuto
    char key = keypad.getKey();
    if (key != NO_KEY) {
      StartGame = true;
    }
  }

  generateSecretCode();  // Genera il codice segreto casuale all'inizio del programma
  displayCode();  // Visualizza il codice segreto iniziale
}

void loop() {
  if (flashScoresScreen) {
    flashScores();
  } else {
    playGame();
  }
}

void flashScores() {
  for (int i = 0; i < 6; ++i) {
    lcd.clear();
    delay(500);
    displayScores();  // Aggiungi questa funzione per visualizzare gli scores
    delay(500);
  }
  flashScoresScreen = false;
}

void displayScores() {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("BEST");
  lcd.setCursor(7, 0);
  lcd.print("3=");
  lcd.print(Score3);
  lcd.setCursor(12, 0);
  lcd.print("4=");
  lcd.print(Score4);
  lcd.setCursor(0, 1);
  lcd.print("SCORES");
  lcd.setCursor(7, 1);
  lcd.print("5=");
  lcd.print(Score5);
  lcd.setCursor(12, 1);
  lcd.print("6=");
  lcd.print(Score6);
}


void playGame() {
  int attempts = 1;

  while (true) {
    char key = keypad.getKey();  // Leggi il carattere dalla tastiera
    if (key) {
      if (key == 'D') {
        lcd.clear();
        lcd.setCursor(0, 0);  // Posizione (0, 1)
        lcd.print("YOU LOSE...");
        lcd.setCursor(12, 0);
        lcd.write((byte)0);
        lcd.setCursor(0, 1);  // Posizione (0, 1)
        lcd.print(code);
        lcd.setCursor(9, 1);  // Posizione (9, 1)
        lcd.print("Try:");
        lcd.print(attempts);
                
        bool ReStartGame = false;
        while (!ReStartGame) {
          // Verifica se un tasto è stato premuto
          char key = keypad.getKey();
          if (key != NO_KEY) {
            ReStartGame = true;
          }
        }

        inputMode = false;
        currentDigit = 0;
        // Inizia una nuova partita
        startNewGame();
        return;
      } else if (isdigit(key) && currentDigit < currentCodeLength) {
        guess[currentDigit] = key;
        currentDigit++;
        lcd.setCursor(currentDigit - 1, 1);  // Posizione (colonna, riga)
        lcd.print(key);
        displayGuess();
      } else if (key == 'B' && currentDigit == currentCodeLength) {
        // Confronta il tentativo con il codice segreto
        int correctPlace = 0;
        int wrongPlace = 0;

        // Array per tenere traccia delle cifre già controllate nel codice segreto
        bool codeDigitUsed[currentCodeLength] = {false};
        bool guessDigitUsed[currentCodeLength] = {false};

        // Confronto per posizione corretta e corretta ma in posizione sbagliata
        for (int i = 0; i < currentCodeLength; i++) {
          if (guess[i] == code[i]) {
            correctPlace++;
            guessDigitUsed[i] = true;
            codeDigitUsed[i] = true;
          }
        }

        // Confronto per posizione sbagliata
        for (int i = 0; i < currentCodeLength; i++) {
          if (!guessDigitUsed[i]) {
            for (int j = 0; j < currentCodeLength; j++) {
              if (!codeDigitUsed[j] && guess[i] == code[j]) {
                wrongPlace++;
                guessDigitUsed[i] = true;
                codeDigitUsed[j] = true;
                break;
              }
            }
          }
        }

        displayIndication(correctPlace, wrongPlace);

        if (correctPlace == currentCodeLength) {
          lcd.clear();
          lcd.setCursor(0, 0);
          lcd.print("YOU WIN!");
          lcd.setCursor(11, 0);  // Posizione (11, 0)
          lcd.write(byte(1));
          lcd.setCursor(12, 0);  // Posizione (12, 0)
          lcd.write((byte)2);
          lcd.setCursor(9, 0);
          lcd.write((byte)3);
          lcd.setCursor(14, 0);  // Posizione (12, 0)
          lcd.print(correctPlace);
          lcd.setCursor(15, 0);
          lcd.print(wrongPlace);
          lcd.setCursor(0, 1);  // Posizione (0, 1
          lcd.print(code);
          lcd.setCursor(9, 1);  // Posizione (9, 1)
          lcd.print("Try:");
          lcd.print(attempts);

// Aggiornamento delle variabili di punteggio solo se il nuovo punteggio è migliore
switch (currentCodeLength) {
  case 3:
    if (Score3 == 50 || attempts < Score3) {
      if (attempts < Score3) {
        Score3 = attempts;
        // Salva il nuovo punteggio nella EEPROM
        EEPROM.put(addressScore3, Score3);
      }
    }
    break;
  case 4:
    if (Score4 == 50 || attempts < Score4) {
      if (attempts < Score4) {
        Score4 = attempts;
        // Salva il nuovo punteggio nella EEPROM
        EEPROM.put(addressScore4, Score4);
      }
    }
    break;
  case 5:
    if (Score5 == 50 || attempts < Score5) {
      if (attempts < Score5) {
        Score5 = attempts;
        // Salva il nuovo punteggio nella EEPROM
        EEPROM.put(addressScore5, Score5);
      }
    }
    break;
  case 6:
    if (Score6 == 50 || attempts < Score6) {
      if (attempts < Score6) {
        Score6 = attempts;
        // Salva il nuovo punteggio nella EEPROM
        EEPROM.put(addressScore6, Score6);
      }
    }
    break;
}


        // Stampa delle variabili di punteggio sul monitor seriale
        Serial.print("Score3: ");
        Serial.println(Score3);
        Serial.print("Score4: ");
        Serial.println(Score4);
        Serial.print("Score5: ");
        Serial.println(Score5);
        Serial.print("Score6: ");
        Serial.println(Score6);

          bool continueGame = false;
          while (!continueGame) {
            // Verifica se un tasto è stato premuto
            char key = keypad.getKey();
            if (key != NO_KEY) {
              continueGame = true;
            }
          }
          inputMode = false;
          currentDigit = 0;
          // Inizia una nuova partita
          startNewGame();
          return;
        }

        currentDigit = 0;
        attempts++;
      } else if (key == 'A') {
        // Aumenta la lunghezza del codice segreto
        currentDigit = 0;
        increaseCodeLength();
        inputMode = false;
        return;
      } else if (key == 'C' && currentDigit > 0) {
        // Backspace per cancellare il carattere precedente
        currentDigit--;
        lcd.setCursor(currentDigit + 7, 1);
        lcd.print('-');
             }else if (key == '*') {
        flashScoresScreen = true;
        flashScores();
        flashScoresScreen = false;
      displayCode();  // Ripristina la visualizzazione del feedback del codice
    }
  
    }
  }
}
